/* $Id: xsh_util_bpmap_nl_raw2pre.c,v 1.1 2010/12/08 15:01:45 amodigli Exp $
 *
 * This file is part of the XSHOOTER Pipeline
 * Copyright (C) 2002,2003 European Southern Observatory
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*
 * $Author: amodigli $
 * $Date: 2010/12/08 15:01:45 $
 * $Revision: 1.1 $
 * $Name: HEAD $
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

/*-----------------------------------------------------------------------------
  Includes
  ----------------------------------------------------------------------------*/
#include <string.h>

/* cpl */
#include <cpl.h>

/* irplib */
#include <xsh_dfs.h>
//#include <xsh_data_pre.h>
#include <xsh_parameters.h>
#include <xsh_utils.h>
#include <xsh_drl.h>
#include <xsh_msg.h>
#include <xsh_pfits.h>
#include <xsh_error.h>


/*-----------------------------------------------------------------------------
  Defines
  ----------------------------------------------------------------------------*/

#define RECIPE_ID "xsh_util_bpmap_nl_raw2pre"
#define RECIPE_AUTHOR "A.Modigliani"
#define RECIPE_CONTACT "Andrea.Modigliani@eso.org"
#define PRO_IMA "PRO_IMA"
#define KEY_VALUE_HPRO_DID                 "PRO-1.15"
/*-----------------------------------------------------------------------------
  Functions prototypes
  ----------------------------------------------------------------------------*/

static int xsh_util_bpmap_nl_raw2pre_create(cpl_plugin *) ;
static int xsh_util_bpmap_nl_raw2pre_exec(cpl_plugin *) ;
static int xsh_util_bpmap_nl_raw2pre_destroy(cpl_plugin *) ;
static int xsh_util_bpmap_nl_raw2pre(cpl_parameterlist *, cpl_frameset *) ;

/*-----------------------------------------------------------------------------
  Static variables
  ----------------------------------------------------------------------------*/

static char 
xsh_util_bpmap_nl_raw2pre_description_short[] = "Converts RAW_BP_MAP_NL to BP_MAP_NL";
static char xsh_util_bpmap_nl_raw2pre_description[] =
  "This recipe converts RAW_BP_MAP_NL to BP_MAP_NL.\n"
  "Information on relevant parameters can be found with\n"
  "esorex --params xsh_util_bpmap_nl_raw2pre\n"
  "esorex --help xsh_util_bpmap_nl_raw2pre\n"
  "\n";

/*-----------------------------------------------------------------------------
  Functions code
  ----------------------------------------------------------------------------*/
/*---------------------------------------------------------------------------*/
/**
 * @defgroup xsh_util_bpmap_nl_raw2pre  Image Shift Along X or Y
 * @ingroup util_recipes
 */
/*---------------------------------------------------------------------------*/

/**@{*/
/*---------------------------------------------------------------------------*/
/**
   @brief    Build the list of available plugins, for this module. 
   @param    list    the plugin list
   @return   0 if everything is ok

   This function is exported.
*/
/*---------------------------------------------------------------------------*/
int cpl_plugin_get_info(cpl_pluginlist * list)
{
  cpl_recipe  *   recipe = cpl_calloc(1, sizeof *recipe ) ;
  cpl_plugin  *   plugin = &recipe->interface ;

  cpl_plugin_init(plugin,
		  CPL_PLUGIN_API,
		  XSH_BINARY_VERSION,
		  CPL_PLUGIN_TYPE_RECIPE,
		  RECIPE_ID,
		  xsh_util_bpmap_nl_raw2pre_description_short,
		  xsh_util_bpmap_nl_raw2pre_description,
		  RECIPE_AUTHOR,
		  RECIPE_CONTACT,
		  xsh_get_license(),
		  xsh_util_bpmap_nl_raw2pre_create,
		  xsh_util_bpmap_nl_raw2pre_exec,
		  xsh_util_bpmap_nl_raw2pre_destroy) ;

  cpl_pluginlist_append(list, plugin) ;
    
  return 0;
}

/*---------------------------------------------------------------------------*/
/**
   @brief    Setup the recipe options    
   @param    plugin  the plugin
   @return   0 if everything is ok

   Create the recipe instance and make it available to the application using 
   the interface. 
*/
/*---------------------------------------------------------------------------*/
static int xsh_util_bpmap_nl_raw2pre_create(cpl_plugin * plugin)
{
  cpl_recipe      * recipe ;
  //cpl_parameter   * p ;

  /* Reset library state */
  xsh_init();

  /* Get the recipe out of the plugin */
  if (cpl_plugin_get_type(plugin) == CPL_PLUGIN_TYPE_RECIPE) 
    recipe = (cpl_recipe *)plugin ;
  else return -1 ;
  cpl_error_reset();


  /* Create the parameters list in the cpl_recipe object */
  recipe->parameters = cpl_parameterlist_new() ; 

  /* Set generic parameters (common to all recipes) */
  check( xsh_parameters_generic( RECIPE_ID, recipe->parameters ) ) ;

 cleanup: 

  /* Return */
  return 0;
}

/*---------------------------------------------------------------------------*/
/**
   @brief    Execute the plugin instance given by the interface
   @param    plugin  the plugin
   @return   0 if everything is ok
*/
/*---------------------------------------------------------------------------*/
static int xsh_util_bpmap_nl_raw2pre_exec(cpl_plugin * plugin)
{
  cpl_recipe  *   recipe ;
  int code=0;
  cpl_errorstate initial_errorstate = cpl_errorstate_get();

  /* Get the recipe out of the plugin */
  if (cpl_plugin_get_type(plugin) == CPL_PLUGIN_TYPE_RECIPE) 
    recipe = (cpl_recipe *)plugin ;
  else return -1 ;
  cpl_error_reset();

  code = xsh_util_bpmap_nl_raw2pre(recipe->parameters, recipe->frames) ;


  if (!cpl_errorstate_is_equal(initial_errorstate)) {                      
    /* Dump the error history since recipe execution start.                
       At this point the recipe cannot recover from the error */           
    cpl_errorstate_dump(initial_errorstate, CPL_FALSE, NULL);              
  } 

  return code ;
}

/*---------------------------------------------------------------------------*/
/**
   @brief    Destroy what has been created by the 'create' function
   @param    plugin  the plugin
   @return   0 if everything is ok
*/
/*---------------------------------------------------------------------------*/
static int xsh_util_bpmap_nl_raw2pre_destroy(cpl_plugin * plugin)
{
  cpl_recipe  *   recipe ;
    
  /* Get the recipe out of the plugin */
  if (cpl_plugin_get_type(plugin) == CPL_PLUGIN_TYPE_RECIPE) 
    recipe = (cpl_recipe *)plugin ;
  else return -1 ;

  cpl_parameterlist_delete(recipe->parameters) ; 
  return 0 ;
}

/*---------------------------------------------------------------------------*/
/**
   @brief    Get the command line options and execute the data reduction
   @param    parlist     the parameters list
   @param    framelist   the frames list
   @return   0 if everything is ok
*/
/*---------------------------------------------------------------------------*/
static int 
xsh_util_bpmap_nl_raw2pre( cpl_parameterlist   *   parlist, 
		    cpl_frameset        *   framelist)
{
  xsh_instrument* instrument=NULL;


  cpl_frame* bpmap_nl=NULL;
  xsh_pre* pre_bp_nl = NULL;
  cpl_frame* bp_nl_frame=NULL;
  char* bp_nl_tag=NULL;
  char* bp_nl_name=NULL;
  const char* pro_catg=NULL;
  const char* recipe_tags[1] = {XSH_RAW_BP_MAP_NL};
  int recipe_tags_size = 1;
  cpl_frameset* raws=NULL;
  cpl_frameset* calib=NULL;


  xsh_msg("Welcome to XSHOOTER Pipeline release %d.%d.%d",
	  XSH_MAJOR_VERSION,XSH_MINOR_VERSION,XSH_MICRO_VERSION);

 check( xsh_begin( framelist, parlist, &instrument, &raws, &calib,
		    recipe_tags, recipe_tags_size,
		    RECIPE_ID, XSH_BINARY_VERSION,
		    xsh_util_bpmap_nl_raw2pre_description_short ) ) ;

  bpmap_nl=xsh_find_frame_with_tag(framelist,XSH_RAW_BP_MAP_NL,instrument);
  check( pre_bp_nl = xsh_pre_create(bpmap_nl , NULL, NULL, instrument,0));
  bp_nl_tag=cpl_sprintf("%s_%s",XSH_BP_MAP_NL,
                        xsh_instrument_arm_tostring( instrument ));

  bp_nl_name=cpl_sprintf("%s.fits",bp_nl_tag);
  check( bp_nl_frame = xsh_pre_save( pre_bp_nl, bp_nl_name,bp_nl_tag,1 ));
  cpl_frameset_insert(framelist, bp_nl_frame);
  check(pro_catg=XSH_GET_TAG_FROM_ARM(XSH_MASTER_BP_MAP,instrument));

  check( xsh_add_product_bpmap(bp_nl_frame, framelist, parlist,
                               RECIPE_ID, instrument, pro_catg ));


 cleanup:

  //xsh_free_frame(&bpmap_nl);
  //xsh_free_frame(&bp_nl_frame);
  xsh_pre_free(&pre_bp_nl); 
  xsh_free(bp_nl_name);
  xsh_free(bp_nl_tag);
  xsh_free_frameset(&raws);
  xsh_free_frameset(&calib);


  xsh_instrument_free(&instrument);

  if (cpl_error_get_code()) {
    return -1 ;
  } else {
    return 0 ;
  }
}
/**@}*/
